{# Returned HTML must be inline to work with SSE #}<div class="dialog-body dialog-body--small"><form class="d-flex flex-column align-items-center" method="dialog"><img width="44" height="44" class="anim-spin icon mb-3" src="/static/icon-update.svg" alt=""><p class="text-center mb-3">{{message}}</p><progress class="mb-3" value="{{percent}}" max="100">{{percent}}%</progress></form></div>