<!DOCTYPE html>
<html>

<head>
    {% block title %}
    <title>Videobox</title>
    {% endblock %}
    {% include "_meta.html" %}
</head>

{% block body %}

<body class="d-flex flex-column align-items-center justify-content-center" style="height: 100vh;">    
    <p class="text-lg mb-4">
        No series found, please update your library.
    </p>
    <button class="btn d-flex align-items-center" type="button" onclick="Videobox.update()">
        <img class="icon mr-2" width="30" height="30" aria-hidden="true" src="/static/icon-update.svg" alt="">Update Now
    </button>
    <dialog id="update-dialog">
        {% include "_update-dialog.html" %}
    </dialog>
</body>

{% endblock %}

</html>